import { index, workingTree } from "./git-utils";
import { PreciseFormatter } from "./precise-formatter";
export interface ModifiedFileConfig {
    fullPath: string;
    gitDirectoryParent: string;
    base: string | null;
    head: string | typeof index | typeof workingTree;
    selectedFormatter: PreciseFormatter<any>;
}
export declare class ModifiedFile {
    private fullPath;
    private pathInGit;
    /**
     * An optional commit SHA pair which will be used to inform how the git
     * commands are run. E.g. `git diff`
     */
    private base;
    private head;
    /**
     * The chosen formatter to be run on the modified file.
     */
    private selectedFormatter;
    /**
     * The parent directory of the relevant .git directory that was resolved
     * for the modified file.
     */
    private gitDirectoryParent;
    /**
     * The contents of the file in their current state on the user's file
     * system
     */
    fileContents: string;
    /**
     * The final file contents, after we've run the formatter
     */
    private formattedFileContents;
    /**
     * The resolved formatter config which applies to this file
     */
    private formatterConfig;
    /**
     * The calculated character ranges which have been modified
     * within this file
     */
    private modifiedCharacterRanges;
    constructor({ fullPath, gitDirectoryParent, base, head, selectedFormatter }: ModifiedFileConfig);
    /**
     * Return true if the whole file has already been formatted appropriately based on
     * the resolved formatter config. We can use this as a check to skip unnecessary work.
     */
    isAlreadyFormatted(): boolean;
    /**
     * Run the formatters check mode on the given ranges and return true if they are all
     * already formatted appropriately based on the resolved formatter config.
     */
    hasValidFormattingForCharacterRanges(): boolean;
    /**
     * Run the formatter on the file contents and store the result
     */
    formatCharacterRangesWithinContents(): void;
    /**
     * Return true if the formatted file contents are different to
     * what was originally resolved from disk.
     */
    shouldContentsBeUpdatedOnDisk(): boolean;
    /**
     * Write the updated file contents back to disk.
     */
    updateFileOnDisk(): void;
    /**
     * We handle errors locally within this method to allow for
     * more granular feedback within the main() function of the
     * library.
     */
    calculateModifiedCharacterRanges(): {
        err: Error | null;
    };
}
